/*
##############################################################
   LC METR
##############################################################
   
   s zobrazenim na LCD, generatorem a komunikaci
   
   (c) Ivo Strasil 2005
   Wurmova 22
   602 00 Brno
   ivo.strasil@centrum.cz   
   
   Tento zdrojovy kod ani s jeho pomoci vytvoreni program 
   NENI volne siritelnym pro komercni vyuziti!
##############################################################

Verze 1.4
=========
OK ... oprava frekvence generatoru pri zmene tlacitky

Verze 1.3
=========
OK ... oprava frekvence PWM, cistsi PWM
OK ... novy kompilator, asm(nop) v DelayUs
OK ... serizeni frekvence generatoru

Verze 1.2
=========
OK ... imp/min dava po RS232 hodnoty


Verze 1.1
=========

OK ... pamet nast. generatoru (M1, M2)
OK ... zmena frekv. generatoru pri behu   
OK ... pri behu generatoru zakaz preruseni
OK ... ihned po nast. vystupnich urovni generatoru je na vystupu uroven log.0
OK ... po startu je na vystupu 0V
OK ... citac meri o 0,05% presneji (oprava konstanty)
OK ... po stisku (+) v rezimu Imp/min se jsou impulsy indikovany pipnutim pri frekvenci nizsi nez 400 imp/min
OK ... kompenzace utlumu antialias filtru


Verze 1.0
=========

OK ... 232 ovladani
OK ... Nacitani prubehu po 232
OK ... Prepinani vstupu pro impcnt/periodu, blok. oscilatoru
OK ... Casovy spinac, timer
OK ... Vystup napeti
OK ... Generatory - prepinani urovni
OK ... Voltmetr
OK ... Generator sumu

OK ... Generator - rozmitani
OK ... Logaritmicke rozmitani

OK ... Nastaveni Cint
c

*/


#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "lcd.h"
#include "a2d.h"



/// DEFINICE HARDWARU
#define 	kalibrace_on()	PORTD|=32;
#define 	kalibrace_off() PORTD&=(unsigned char)~32;

#define TL_OK		(!(PINC&128))
#define TL_ESC		(!(PINC&64))
#define TL_PLUS 	(!(PINC&16))
#define TL_MINUS	(!(PINC&32))
#define TL_DALSI	(!(PINC&8))
#define TL_LX		(!(PINC&4))
#define TL_CX		((PINC&4)&&(PINC&8))

#define DIN0		(PINA&8)
#define DIN1		(PINA&4)

#define rozsah0_primo() 	PORTA&=~(unsigned char)2;DDRA&=~(unsigned char)2;
#define rozsah0_polovina()	PORTA&=~(unsigned char)2;DDRA|=2;
#define rozsah0_plus()	    PORTA|=(unsigned char)2;DDRA|=2;


#define rozsah1_primo() 	PORTA&=~(unsigned char)1;DDRA&=~(unsigned char)1;
#define rozsah1_polovina()	PORTA&=~(unsigned char)1;DDRA|=1;
#define rozsah1_plus()	    PORTA|=(unsigned char)1;DDRA|=1;

#define filter_on()			PORTA&=~(unsigned char)1;DDRA|=1;
#define filter_off()		PORTA&=~(unsigned char)1;DDRA&=~(unsigned char)1;

//#define LX			(!(PINC&8))


#define SYSCLK 16

//#define MER_LC		0
#define MER_VOLT	1
#define MER_232 	2
#define MER_VOUT	3
#define MER_TIMER	4
#define MER_OSC 	5
#define MER_CITAC	6
#define MER_PERIODA	7
#define MER_RPM 	8
#define MER_GEN 	0
#define MER_DDS 	9
#define MER_DDS_1 	10
#define MER_DDS_2 	11
#define MER_MAX 	11


void BeepLowMs (unsigned int c)
{
 while (c) {
  c--;
  PORTB&=~(unsigned char)1;
  DelayUs(250);
  DelayUs(250);
  PORTB|=1;
  DelayUs(250);
  DelayUs(250);
 }
 
}
void BeepHighMs (unsigned int c)
{ 
 c<<=1;
 while (c) {
  c--;
  PORTB&=~(unsigned char)1;
  DelayUs(250);
  PORTB|=1;
  DelayUs(250);
 } 
}

void DelayBtnMs (unsigned int c)
{
 while (c&&!TL_OK&!TL_ESC) {
  c--;
  DelayUs(250);
  DelayUs(250);
  DelayUs(250);
  DelayUs(250);
 }
 
}

void uart_init(unsigned int baud)
{
 UBRRH=(unsigned char)(baud>>8);
 UBRRL=(unsigned char)(baud);
 UCSRA=(1<<UDRE)+2;
 UCSRB=(1<<RXEN)|(1<<TXEN)|(1<<RXCIE);
 UCSRC=128+6;
}

int uart_transmit(char c)
{
 if (c == '\n')
   uart_transmit('\r'); 
 while ( !( UCSRA & (1<<UDRE)) )
  ;
 UDR = c;
 return 0;
}

volatile unsigned char CmdBuf[21];
volatile unsigned char CmdPos=0,CmdFlag=0,interruptflag=0;

SIGNAL (SIG_UART_RECV) {
 unsigned char data=UDR;

 if (CmdFlag&&data==27) {
  interruptflag=1;
 }
 if (data==13||data==10||CmdFlag) {
  CmdFlag=1;
 } else {
  if (CmdPos<20&&data!=27) { //ignoruj prebytecne [ESC]
   CmdBuf[CmdPos]=data; 
   CmdPos++;
   interruptflag=0;
  } 
 } 
}

void CommandDone();
void CommandDone() {
 for (unsigned char i=0;i<20;i++) {
  CmdBuf[i]=0x00;
 }
 CmdPos=0;
 CmdFlag=0;
 interruptflag=0;
}

unsigned short from,length;

void DecodeLF();
void DecodeLF()
{
      unsigned char k=3;
	  from=0;
	  while (isdigit(CmdBuf[k])&&k<18) {
	   from*=10;
	   from+=CmdBuf[k]-'0';
	   k++;
	  }
	  k++;
	  length=0;
	  while (isdigit(CmdBuf[k])&&k<19) {
	   length*=10;
	   length+=CmdBuf[k]-'0';
	   k++;
	  }
};




unsigned char ReceiveByte( void )
{
	while ( !(UCSRA & (1<<RXC)) );	
	return UDR;
}


FILE * lcd; FILE * seriak;

volatile unsigned long T1CNT_high;
volatile unsigned char seccnt;

SIGNAL(SIG_OVERFLOW1)
{
 //zakaz preruseni
 
 T1CNT_high++;
 //povol preruseni
 
}

volatile unsigned char zkrac=5;

SIGNAL(SIG_OVERFLOW0)
{
 //zakaz preruseni
 
 //zkraceni na 5 cyklu
 TCNT0=zkrac;
 if (seccnt) {seccnt--;}
 //povol preruseni
 
}

unsigned long citac(void) {
  TCNT1=0;
  T1CNT_high=0;
  TCNT0=zkrac;
  seccnt=249;
  while (seccnt) {};
  return (unsigned long)TCNT1+(unsigned long)((unsigned long)256*(unsigned long)256*T1CNT_high);
}

float ckal;

unsigned long fklid,fkal=0;
float ci,li;
signed long cipom,lipom;
 
void cekejte(); 
void cekejte() {
     lcd_clear();
     printf_P(PSTR("Pros\x04m, \x05""ekejte"));
     lcd_putcommand(0xC0);	
}
void Cek5s();
void Cek5s() {
 for (unsigned char i=0;i<5;i++) {
  DelayMs(1000);
  lcd_putch(165);
 } 
 
}

void Kalibrace();
void Kalibrace()
{
 //autokalibrace
 while (!TL_OK&&!(TL_ESC&&(fkal!=0))&&!TL_DALSI) {
 lcd_clear(); 
 if (TL_LX) {
  printf_P(PSTR("Zkratujte svorky"));  
 } else {
  printf_P(PSTR("Rozpojte svorky"));  
 } 
 lcd_putcommand(0xC0);
 printf_P(PSTR("Stiskn\x06te tl.OK "));  
 DelayBtnMs(500);
 lcd_putcommand(0xC0);
 printf_P(PSTR("Kalibrace      "));
 DelayBtnMs(500);  
 };
 while (TL_ESC) {DelayMs(10);};
 if (TL_OK||(fkal==0&&!TL_DALSI)) {
 cekejte();

 lcd_putch(165);
 kalibrace_on();
 //ustaleni
 unsigned char i;
 Cek5s();
 //mereni kal. kmitoctu
 fkal=citac();
 kalibrace_off(); 
 //mereni klid. kmitoctu
 Cek5s();
 fklid=citac(); 
 lcd_clear();
 
 
 
 printf_P(PSTR("fi  =%lu Hz"),fklid);
 lcd_putcommand(0xC0);
 printf_P(PSTR("fkal=%lu Hz"),fkal); 
 DelayMs(500);
 
 ci=ckal*((float)fkal*(float)fkal)/((float)fklid*(float)fklid-(float)fkal*(float)fkal);
 li=(float)1.0/((float)4*ci*9.8696044010893586188344909998762*(float)fklid*(float)fklid);
 
 cipom=(signed long)floor(ci*(float)1e12); //pF
 lipom=(signed long)floor(li*(float)1e9); //nH
  
 lcd_clear();
 printf_P(PSTR("Ci %ld pF"),cipom);
 lcd_putcommand(0xC0);
 printf_P(PSTR("Li %ld nH"),lipom);
 DelayMs(950);
 
 BeepLowMs(50);
 };
}

void chyba();
void chyba() {
	lcd_clear();
    printf_P(PSTR("CHYBA"));
	lcd_putcommand(0xC0);
	printf_P(PSTR("Obvod nekmita."));
}

void presnost() {
	lcd_clear();
    printf_P(PSTR("Mozne snizeni"));
	lcd_putcommand(0xC0);
	printf_P(PSTR("presnosti!"));
	DelayMs(250);
	lcd_clear();
}


float fp;	

#define AH_CATCH	2

void LCMetr();
void LCMetr()
{
   unsigned char ahold=0;
   if (!fkal) {
    Kalibrace();
   };
   cekejte();
   float fpsrov=0;
   while (!TL_DALSI) {
    unsigned long fakt;
    
    fakt=citac();
	if (TL_OK&&ahold) {
	 fpsrov=0;
	 ahold=1;
	};
    //autohold 1-0
	if (TL_MINUS) {
	 ahold=!ahold;
	 fpsrov=0;	  
	};	 
	if (fakt>20||ahold==AH_CATCH) {
	 if (ahold!=AH_CATCH) {
      fp=((float)fklid*(float)fklid/(float)fakt/(float)fakt-1.0);	
	 }; 
	 lcd_clear();
	 if (TL_OK&&!ahold) {	  
	   if ((float)fpsrov>0.01) {
	    fpsrov=0;
	   } else {	   
	    fpsrov=fp;
	   };	  
	 };
	 if (TL_LX) {
	  printf_P(PSTR("Lx ="));
	 } else {
	  printf_P(PSTR("Cx ="));
	 };
     lcd_putcommand(0x88);	       
	 if (!ahold&&fpsrov>0.01&&(fpsrov*1.25>fp&&fpsrov*0.75<fp)) {
	   
	   signed long delta=(signed int)((10000.0*fp/fpsrov)-10000.0);
	   if ((signed int)delta<0) {delta=-delta;lcd_putch('-');} else {lcd_putch('+');};
	   printf_P(PSTR("%d,"),delta/100);	  
	   printf_P(PSTR("%.2d"),delta%100);
	   lcd_putch('%');
	 };
	 if (ahold) {
	  if ((fpsrov*1.02>fp&&fpsrov*0.98<fp&&fpsrov>0.015)||ahold==AH_CATCH) {
	   printf_P(PSTR("AUTOHOLD"));
	   if (ahold!=AH_CATCH) {
	    BeepLowMs(50);
		BeepHighMs(50);
		BeepLowMs(50);
	   }
	   ahold=AH_CATCH;
       fp=fpsrov;
	   
	  } else {
	   //neni
	   printf_P(PSTR("AH zap"));
	   fpsrov=fp;
	  }	  
	 };
	 lcd_putcommand(0xC0);
	 if (TL_LX) {
	  fp=li*fp;
	  lipom=(signed long)floor(fp*(float)1e9); //nH      
	  if ((signed long)lipom<(signed long)0) {lipom=-lipom;lcd_putch('-');};
	  if (lipom>1000000) {
	   printf_P(PSTR("%ld,%.2d mH"),lipom/1000000,(lipom%1000000)/10000);
	   if (lipom>75000000) {
		DelayMs(700);
		presnost();
	   }
	  } else {
	   printf_P(PSTR("%ld,%.2d uH"),lipom/1000,(lipom%1000)/10);
      }   
	  } else {
	  fp=ci*fp;
      cipom=(signed long)floor(fp*(float)1e12); //pF	  
	  if (cipom<1000) {
       printf_P(PSTR("%ldpF"),cipom);
	  } else {
	   if (cipom<500000) {
	    printf_P(PSTR("%ld,%.3d nF"),(signed long)cipom/1000,cipom%1000);
	   } else {
	    printf_P(PSTR("%ld,%.3d uF"),(signed long)cipom/1000000,(cipom%1000000)/1000);
		DelayMs(700);
		presnost();
       };	   
      };	 
     };
    } else {
	 chyba(); //kdyz nekmita
    }	
	if (TL_ESC) {
	 while (TL_ESC) {DelayMs(15);BeepLowMs(5);};
	 fklid=fakt;
	 Kalibrace();
	};
	if (TL_PLUS&&TL_LX) {
	 unsigned long fkp;
	 //mereni Lx metodou bez zanedbani R(L)
     cekejte();
     //mereni klid. kmitoctu
     DelayMs(1000);
     lcd_putch(165);
     DelayMs(1000);
     lcd_putch(165);
     fkp=citac();
     lcd_putch(165);
     kalibrace_on();
     //ustaleni
	 
     DelayMs(1500);
     lcd_putch(165);
     DelayMs(1500);
     lcd_putch(165);
     DelayMs(1500);
     lcd_putch(165);
     DelayMs(1500);
     lcd_putch(165);
     //mereni kal. kmitoctu
     fkal=citac(); 
     lcd_clear();
	 BeepLowMs(50);
     kalibrace_off();  
	 if (fkal>50&&fkp>50) {  
      //vypocet 
	  fp=ckal/(ci*4.0*3.1415926535897932384626433832795*3.1415926535897932384626433832795*(ci+ckal)*(square((float)fkp)-square((float)fkal)));
	  lcd_clear();
      lipom=(signed long)floor((fp-li)*(float)1e9); //nH	 
	  printf_P(PSTR("Lx = "));	
	  if (lipom>1000) {
       printf_P(PSTR("%ld,%.2d uH"),lipom/1000,(lipom%1000)/10);
	  } else {
	   printf_P(PSTR("%ld nH"),lipom);
	  };
	  lcd_putcommand(0xC0);
	  printf_P(PSTR("R  = "));	
	  fp=sqrt(fp*(1/ci-fp*4.0*3.1415926535897932384626433832795*3.1415926535897932384626433832795*square((float)fkp)));
      lipom=(signed long)floor(fp); //nH	 
	  if (lipom>=0&&lipom<1000) {
	   fprintf(lcd,("%ld ohm"),lipom);
	  } else {
	   printf_P(PSTR("nelze ur\x05it"),lipom);
      }   
	 } else {
	  chyba();
     }	 
	 while (!TL_OK&&!TL_ESC&&!TL_DALSI) {};
	 lcd_clear();
	 while (TL_OK||TL_ESC) {DelayMs(50);};BeepHighMs(50);
	};
   };	
};   

#define DA_INP_BUF		64
#define DA_GAIN_1		32
#define DA_GAIN_2		0
#define DA_ON			16

#define DA_REZIM_05		0
#define DA_REZIM_010	1
#define DA_REZIM_55		2


unsigned char DAConf=DA_INP_BUF+DA_GAIN_2+DA_ON;

void dasend(unsigned int value) {
/* Start transmission */
while(!(SPSR & (1<<SPIF)))
;
PORTB|=16;
PORTB&=(unsigned char)~16;
SPDR = (DA_INP_BUF+DA_GAIN_2+DA_ON) | (value>>8);
/* Wait for transmission complete */
while(!(SPSR & (1<<SPIF)))
;
SPDR = value & 0xFF;
}

void dasendAsync(unsigned int value) {
/* Start transmission */
SPDR = DAConf | (value>>8);
/* Wait for transmission complete */
while(!(SPSR & (1<<SPIF)))
;
SPDR = value & 0xFF;
while(!(SPSR & (1<<SPIF)))
;
PORTB|=16;
PORTB&=(unsigned char)~16;
}


void Diag();
void Diag() {
 while (1) {
  lcd_putcommand(0xC0);  
  if (TL_OK) {lcd_putch('O');};
  if (TL_ESC) {lcd_putch('E');};
  if (TL_PLUS) {lcd_putch('+');kalibrace_on();};
  if (TL_MINUS) {lcd_putch('-');kalibrace_off();};
  if (TL_LX) {lcd_putch('L');};
  if (TL_CX) {lcd_putch('C');};
  if (TL_DALSI) {lcd_putch('D');};  
  lcd_putch(' ');lcd_putch(' ');lcd_putch(' ');lcd_putch(' ');
  lcd_putcommand(0x80);
  printf("%7lu Hz",citac());  
 }; 
};

#define GI_ALLOW_MINUS	1
#define GI_ING			2
#define GI_TIME 		4
#define GI_HZ			8
#define GI_S			16
#define GI_V		32
#define GI_LONGTIME	64
#define GI_FULLPR		128

//funkce nacte integer hodnotu 



unsigned long GetInt(unsigned long def, unsigned char flags, unsigned long maximum) {
 unsigned long pom=def;
 unsigned char autorep=0;
 unsigned long rad=1;
 while (TL_OK||TL_ESC) {};
 DelayMs(50);
 do {
  lcd_putcommand(0xC0);
  //vypis v inzenyrskem formatu
  if (flags&GI_LONGTIME) {
   if (pom>=3600) {
    printf_P(PSTR("%lu hod %l.2u min"),(unsigned long)(pom/3600),(pom/60)%60);
	pom=(pom/60)*60;
   } else
   {
    if (pom>=60) {
     printf_P(PSTR("%lu:%l.2u"),(unsigned long)pom/60,pom%60);
    } else {
     printf_P(PSTR("%lu s"),(unsigned long)pom);
    };
   };	
  } else
  if (flags&GI_ING) {
   if (pom>1000) {
    printf_P(PSTR("%lu,%l.3u k"),(unsigned long)pom/1000,pom%1000);
   } else {
    printf_P(PSTR("%lu "),(unsigned long)pom);
   };
  } else 
  if (flags&GI_S) {
   if (pom>1000000) {
    printf_P(PSTR("%lu,%l.3u "),(unsigned long)pom/16000000,(pom/16000)%1000);
   } else 
   if (pom>1000) {
    printf_P(PSTR("%lu,%l.3u m"),(unsigned long)pom/16000,(pom/16)%1000);
   } else {
    printf_P(PSTR("%lu u"),(unsigned long)pom/16);
   };
  } else
  if (flags&GI_V) {
   if (pom>=500) {
    printf_P(PSTR("+%2u"),(pom-500)/100);
	printf_P(PSTR(",%.2u V"),(pom-500)%100);
   } else {
    printf_P(PSTR("-%u"),(500-pom)/100);
	printf_P(PSTR(",%.2u V"),(500-pom)%100);
   };
  }
  else
  {
  //normalni vypis
   printf_P(PSTR("%lu "),pom);
  };	
  if ((flags&GI_HZ)&&!(flags&GI_V)) {
   lcd_putch('H');
   lcd_putch('z');
  };
  if (flags&GI_S) {
   lcd_putch('s');
  };
  lcd_putch(' ');
  lcd_putch(' ');
  lcd_putch(' ');
  //modifikace hodnoty
  if (!TL_MINUS&&!TL_PLUS) 
  { 
	autorep=0;
    rad=10000000;
	while (rad>1&&rad>pom/100) {
	 rad=rad/10;
	};
	if (flags&GI_S) {rad*=16;};	
	if (  ((!(flags&GI_FULLPR)&&!(flags&(GI_HZ|GI_V)))&&(!(flags&GI_S||flags&GI_V||flags&GI_LONGTIME)||(TL_ESC))) || (GI_HZ&&TL_ESC)) {
	 pom=pom/rad;
	 pom*=rad;    
	};	
	if (TL_ESC&&(flags&GI_V)&&!(flags&GI_HZ)) {
	 pom=500;
	};
  };

  while (pom&&rad>pom) {
   rad=rad/10;
  };
  if ((flags&GI_FULLPR)&&(!TL_MINUS)&&(!TL_PLUS)) {
   rad=1;
  };
  if (TL_MINUS&&pom>=rad) {
   if (!autorep) {
    DelayMs(250);
    if (TL_MINUS) {
     autorep=1;  
    } else {
     pom-=rad;
    }	
   } else {
    if (autorep<199) {
     autorep++;
	 pom-=rad;	 	 
	} else {
	pom-=rad;
	 rad=rad*10;
	 autorep=1;
	 
	}	
   };	
  };
  if (TL_PLUS&&(((unsigned long)pom+rad<2000000)||((flags&GI_S)&&((unsigned long)pom+(unsigned long)rad<(unsigned long)4200000000)))) {
   if (!autorep) {
    DelayMs(250);
    if (TL_PLUS) {
     autorep=1;
    } else {
     pom+=rad;
    };	 
   } else {
    if (autorep<99) {
     autorep++;
	 pom+=rad;	 
	} else {
	 pom+=rad;
	 rad=rad*10;
	 autorep=1;	
	}	
   };	
  };
  if ((flags&GI_V)&&pom>1500) {
   pom=1500;
  };
  if (maximum&&pom>maximum) {
   pom=maximum;
  }
  if ((flags&GI_LONGTIME)&&pom>(unsigned long)3600*99) {
   pom=(unsigned long)3600*99;
  };
  DelayMs(15);
 } while ((!TL_OK)&&(!TL_ESC||!((flags&GI_HZ)&&(flags&GI_V)))&&(TL_DALSI||(flags&GI_FULLPR)));
 lcd_clear();
 while (TL_OK) {};  
 BeepLowMs(50);
 return pom; 
}



unsigned char Opakovat() {
 lcd_clear();
 BeepLowMs(50);
 while (TL_OK||TL_ESC) {DelayMs(10);}; 
 printf_P(PSTR("OK > parametry"));
 lcd_putcommand(0xC0);
 printf_P(PSTR("ESC> jine m\x06ren\x07"));
 while (TL_DALSI&&!TL_OK&&!TL_ESC) {}
 while (TL_OK) {}; 
 BeepLowMs(50);
 return !TL_ESC&&TL_DALSI;
};

unsigned int DAPrepocet(unsigned char rozsah,unsigned int vstup) {
 unsigned long pom;
 if (rozsah==DA_REZIM_05) {
  if (vstup>=1000) {vstup=999;};
  pom=((unsigned long)(vstup-500)*8192/1000);
 } else if (rozsah==DA_REZIM_010) {
  if (vstup>=1500) {vstup=1499;};
  pom=((unsigned long)(vstup-500)*4096/1000);
 } else {
  if (vstup>=1000) {vstup=999;};
  pom=((unsigned long)(vstup)*4096/1000);
 } 
 return (unsigned int)pom;
};

void SetDAR(unsigned char dar) {
 if (dar==DA_REZIM_05) {
  DDRC&=~(unsigned char)2;
 } else if (dar==DA_REZIM_010) {
  DDRC|=2;
  PORTC&=~(unsigned char)2;
 } else {
  DDRC|=2;
  PORTC|=2;
 };
};

//generator impulsu a obdelnikovych prubehu s promennou stridou
void ObdGen(unsigned char jetimer) {
 //moznosti generatoru:
 //spousteni: opakovane | ext. trigger | ext. vstup povoluje beh | tlacitkem
 //vystupni urovne: low, high, cekani pred/po impulsech (low/high/nastaveni)
 //prednastaveni urovni: 3V, 5V, 1V AC, 0,775V AC
 //metoda nastaveni: frekvence | frekvence+% | podrobne
 //generator: pocet impulsu, trvani 0, trvani 1
#define SP_OPAK 	0
#define SP_ALLOW	1
#define SP_TRIG 	2
#define SP_BUTTON	3
#define SP_PWM		4
#define SP_ODPOCET	5


#define UR_3V		0
#define UR_5V		1
#define UR_1V		2
#define UR_0V7		3
#define UR_SET		4

#define TS_FREK 	0
#define TS_PWM		1
#define TS_PODR 	2

#define OG_EEPROM_START	100

 unsigned char spoust,pom=0,pomer=50;
 unsigned int	voltage_low=0, voltage_high=4000, voltage_prewait=500, voltage_postwait=500;
 unsigned long impcnt=100,i,perioda,casodp=60;
 unsigned long prewait=(unsigned long)0,postwait=(unsigned long)0,lowwait=300*16,highwait=150*16;
 unsigned int vol_low,vol_high,vol_pre,vol_post;
 
 TCCR1B=0x01;
 do {
 rozsah0_primo();
 filter_off();
 //nastaveni
 // 1. krok: nastaveni typu generatoru 
 
 if (!jetimer) {
  spoust=eeprom_read_byte(OG_EEPROM_START);
  do {
   lcd_clear();
   printf_P(PSTR("Typ:"));
   lcd_putcommand(0xC0);
   if (spoust==SP_OPAK) {printf_P(PSTR("AKO"));};
   if (spoust==SP_ALLOW) {printf_P(PSTR("AKO s povolen\x04m"));};
   if (spoust==SP_TRIG) {printf_P(PSTR("MKO"));};
   if (spoust==SP_BUTTON) {printf_P(PSTR("Ru\x05n\x07 start"));};
   if (spoust==SP_PWM) {printf_P(PSTR("PWM"));};
   if (spoust==SP_ODPOCET) {printf_P(PSTR("Odpo\x05""et"));};  
   if (TL_PLUS&&spoust<5) {spoust++;};
   if (TL_MINUS&&spoust) {spoust--;};
   while (TL_PLUS||TL_MINUS) {};
   DelayMs(100);
  } while (!TL_OK&&!TL_ESC&&TL_DALSI); 
  while (TL_OK||TL_ESC) {};
  eeprom_write_byte(OG_EEPROM_START,spoust);
 } else {
  spoust=SP_ODPOCET;
 }; 
 // 2. krok: nastaveni urovni 
 pom=eeprom_read_byte(OG_EEPROM_START+1);
 do {
  lcd_clear();
  printf_P(PSTR("Urovne:"));
  lcd_putcommand(0xC0);  
  if (pom==UR_3V) {printf_P(PSTR("CMOS 3V/TTL"));voltage_low=500;voltage_high=800;};
  if (pom==UR_5V) {printf_P(PSTR("CMOS 5V"));voltage_low=500;voltage_high=1000;};
  if (pom==UR_1V) {printf_P(PSTR("+/-1,0V"));voltage_low=400;voltage_high=600;};
  if (pom==UR_0V7) {printf_P(PSTR("+/-0,7V"));voltage_low=423;voltage_high=577;};
  if (pom==UR_SET) {printf_P(PSTR("nastavit"));if (voltage_low==423) {voltage_low=500;voltage_high=800;}};
  if (TL_PLUS&&pom<4) {pom++;};
  if (TL_MINUS&&pom) {pom--;};
  while (TL_PLUS||TL_MINUS) {};
  DelayMs(100);
 } while (!TL_OK&&!TL_ESC&&TL_DALSI); 
 while (TL_OK||TL_ESC) {};
 eeprom_write_byte(OG_EEPROM_START+1,pom);
 // 2b. krok - podrobne nastaveni urovni
 unsigned char darozsah;
 do {
  darozsah=DA_REZIM_05;
  if (pom==UR_SET) {
   lcd_clear();
   printf_P(PSTR("Nap\x06t\x04 LOW:"));
   voltage_low=GetInt(500,GI_V,0);
   printf_P(PSTR("Nap\x06t\x04 HIGH:"));
   voltage_high=GetInt(1000,GI_V,0);  
   if (spoust!=SP_PWM&&spoust!=SP_ODPOCET) {
    printf_P(PSTR("Nap\x06t\x04 PREwait:"));
    voltage_prewait=GetInt(voltage_low,GI_V,0);  
    printf_P(PSTR("Nap\x06t\x04 POSTwait:"));
    voltage_postwait=GetInt(voltage_low,GI_V,0);  
   };	
  }
  //prepocet urovni
  // 1. krok - urceni rozsahu vystupu (0-5/0-10V nebo +-5V)
 
  if (voltage_prewait<500) {darozsah|=DA_REZIM_55;};
  if (voltage_postwait<500) {darozsah|=DA_REZIM_55;};
  if (voltage_low<500) {darozsah|=DA_REZIM_55;};
  if (voltage_high<500) {darozsah|=DA_REZIM_55;};
 
  if (voltage_prewait>1000) {darozsah|=DA_REZIM_010;};
  if (voltage_postwait>1000) {darozsah|=DA_REZIM_010;};
  if (voltage_low>1000) {darozsah|=DA_REZIM_010;};
  if (voltage_high>1000) {darozsah|=DA_REZIM_010;};
  if (voltage_high>1000) {darozsah|=DA_REZIM_010;};
  if (darozsah==DA_REZIM_010+DA_REZIM_55) {
   printf_P(PSTR("CHYBA max.vystup"));
   lcd_putcommand(0xC0);
   printf_P(PSTR("-+5V NEBO 0..10V"));
   DelayMs(2000);
  }; 
 } while (darozsah==DA_REZIM_010+DA_REZIM_55&&TL_DALSI);
 // 2. krok - vypoctu hodnot pro DA
 //lcd_clear();
 //printf_P(PSTR("DAR %u"),darozsah);
 SetDAR(darozsah);
 //DelayMs(200); 
 
 vol_high=DAPrepocet(darozsah,voltage_high);
 vol_low =DAPrepocet(darozsah,voltage_low);
 vol_pre =DAPrepocet(darozsah,voltage_prewait);
 vol_post=DAPrepocet(darozsah,voltage_postwait);
 
 dasendAsync(voltage_low);

 // 3. krok - nastaveni frekvence - pak jen odpotvrzovat delky
 lcd_clear();
 if (spoust==SP_ODPOCET) {
  printf_P(PSTR("Zadej \x05""as:"));
  casodp=GetInt(eeprom_read_word(OG_EEPROM_START+2),GI_LONGTIME,0); 
  if ((unsigned long)casodp<(unsigned long)65000) {
   eeprom_write_word(OG_EEPROM_START+2,(unsigned int)((unsigned long)casodp&0xFFFF));
  } 
  //eeprom_write_word(OG_EEPROM_START+4,(unsigned int)((unsigned long)casodp>>16));
  highwait=0;
  lowwait=0;
  prewait=0;
  postwait=0;
  impcnt=0;
 } else {
  printf_P(PSTR("Frekvence:"));
  lowwait=GetInt(eeprom_read_word(OG_EEPROM_START+6),GI_ING|GI_HZ,60000); 
  eeprom_write_word(OG_EEPROM_START+6,lowwait);
  if (!lowwait) {lowwait=1;};
  if (spoust==SP_PWM) {
   prewait=0;
   postwait=0;
   impcnt=10+lowwait/100;
   perioda=(unsigned long)16000000/lowwait;
   lowwait=perioda/2;
   highwait=perioda/2;
  } else {
   lowwait=(unsigned long)8000000/lowwait; //prevest na us/16
   highwait=lowwait;
   //potvrdit cas LOW
   
   printf_P(PSTR("Trvani LOW:"));
   lowwait=GetInt(lowwait,GI_S,0);
   if (lowwait<16*7) {lowwait=16*7;};
   lowwait-=16*7;
  
   printf_P(PSTR("Trvani HIGH:"));
   highwait=GetInt(highwait,GI_S,0);
   if (highwait<16*7) {highwait=16*7;};
   highwait-=16*7;
  
   printf_P(PSTR("Trvani PREwait:"));
   prewait=GetInt(prewait,GI_S,0);
   if (prewait<16*7) {prewait=16*7;};
   prewait-=16*7;
  
   printf_P(PSTR("Trvani POSTwait:"));
   postwait=GetInt(postwait,GI_S,0);
   if (postwait<16*7) {postwait=16*7;};
   postwait-=16*7;
  
   printf_P(PSTR("Impulsu v s\x06rii:"));
   impcnt=GetInt(impcnt,0,0);
  }; 
 }; 
 
 
 
 
 
 //vypis na displej
 lcd_clear();
 printf_P(PSTR("== Gener\x01tor =="));
   if (spoust==SP_PWM) {
	    lcd_putcommand(0xC0);
		printf_P(PSTR("PWM %.3d %%"),pomer);
lowwait=perioda*(100-pomer)/100;
	if (lowwait>250) {lowwait-=230;};
    highwait=perioda*(pomer)/100;		
	}
 
 //generovani
 while (!TL_ESC&&TL_DALSI) {
  //trigger
  if (spoust) {
   if (spoust==SP_PWM) {
    dasendAsync(voltage_low);
    if (TL_PLUS&&pomer<100)  {pomer++;
	    lcd_putcommand(0xC0);
		printf_P(PSTR("PWM %.3d %%"),pomer);
lowwait=perioda*(100-pomer)/100;
	if (lowwait>250) {lowwait-=230;};
    highwait=perioda*(pomer)/100;		
	};
    if (TL_MINUS&&pomer)  {pomer--;
	    lcd_putcommand(0xC0);
		printf_P(PSTR("PWM %.3d %%"),pomer);
lowwait=perioda*(100-pomer)/100;
	if (lowwait>250) {lowwait-=230;};
    highwait=perioda*(pomer)/100;		
	};
    
    
   };
   if (spoust==SP_BUTTON) {
    dasendAsync(vol_post);
    while (TL_OK) {};DelayMs(100);
    lcd_putcommand(0xC0);
	printf_P(PSTR("Start tla\x05. OK"),pomer);
	while (!TL_OK&&TL_DALSI&&!TL_ESC) {	 
	};
	lcd_putcommand(0xC0);
	printf_P(PSTR("Generuji..."),pomer);	
   }
   if (spoust==SP_TRIG) {
    while(DIN0&&TL_DALSI&&!TL_ESC) {};
    while(!DIN0&&TL_DALSI&&!TL_ESC) {};
   };
   if (spoust==SP_ALLOW) {
    while(!DIN0&&TL_DALSI&&!TL_ESC) {};
   };
   if (spoust==SP_ODPOCET) {
    dasendAsync(vol_low);
    while (TL_OK) {};DelayMs(100);
    lcd_clear();
	printf_P(PSTR("Start tla\x05. OK"));
	while (!TL_OK&&TL_DALSI&&!TL_ESC) {	 
	};
	DelayMs(10);
	while (TL_OK) {};
	DelayMs(100);
	lcd_clear();
	printf_P(PSTR("Odpo\x05""et..."));	
	dasendAsync(vol_high);
	highwait=casodp;
	do {
	 TCNT1=0;
	 T1CNT_high=0;
	 lcd_putcommand(0xC0);
	 printf_P(PSTR("%.2lu:%.2lu:%.2lu"),highwait/3600,(highwait/60)%60,highwait%60);	
	 highwait--;
	 while (TL_DALSI&&!TL_ESC&&!TL_OK&&(TCNT1<(16000000&0xFFFF)||T1CNT_high<(16000000>>16))) {};
	 if (TL_PLUS) {highwait+=60;};
	 if (TL_MINUS&&highwait>=60) {highwait-=60;};
	} while (highwait&&TL_DALSI&&!TL_ESC&&!TL_OK);
	while (TL_OK) {};
	//DelayMs(100);
	//hotovo
	lcd_clear();
	printf_P(PSTR(" === HOTOVO ==="));	
	while (!TL_OK&&!TL_ESC) {
	 BeepLowMs(100);
	 BeepHighMs(100);
	} ;
	DelayMs(20);
	while (TL_OK) {};
	DelayMs(20);
   };
  };
  if (prewait) {
   //prewait
   dasendAsync(vol_pre);
   //cekani  
   TCNT1=0;
   T1CNT_high=0;
   while (TCNT1<(prewait&0xFFFF)||T1CNT_high<(prewait>>16)) {
   }
  }; 
  //impulsy
  for (i=0;i<impcnt&&TL_DALSI&&!TL_ESC;i++) {
   //LOW
   dasendAsync(vol_low);
   //cekani  
   TCNT1=0;
   T1CNT_high=0;
   while (TCNT1<(lowwait&0xFFFF)||T1CNT_high<(lowwait>>16)) {
   }
   //HIGH
   dasendAsync(vol_high);
   TCNT1=0;
   T1CNT_high=0;
   while (TCNT1<(highwait&0xFFFF)||T1CNT_high<(highwait>>16)) {
   }
  }
  if (postwait) {
   //postwait
   dasendAsync(vol_post);
   TCNT1=0;
   T1CNT_high=0;
   while (TCNT1<(postwait&0xFFFF)||T1CNT_high<(postwait>>16)) {
   }  
  }
 }
 } while (Opakovat());
 TCCR1B=0x07;
}


unsigned long rndget();    /* returns a uniformly random 32-bit integer */
void initrnd( /*unsigned long */);  /* initialize the generator */

/* return a random float >= 0 and < 1 */
#define rand_float	    ((float)rndget() / 4294967296.0)

static unsigned long a=1791398085;
/* Choose a value for a from this list
   1791398085 1929682203 1683268614 1965537969 1675393560
   1967773755 1517746329 1447497129 1655692410 1606218150
   2051013963 1075433238 1557985959 1781943330 1893513180
   1631296680 2131995753 2083801278 1873196400 1554115554
*/
static unsigned long x=30903, c=0, ah, al;

void initrnd();
void initrnd( /*unsigned long ci */)
   {
   //if (xi != 0) x = xi;
   //c = ci;
   ah = a>>16;
   al = a&65535;
   }


unsigned long rndget()
   {
   unsigned long xh = x>>16, xl = x & 65535;

   x = x * a + c;
   c = xh*ah + ((xh*al) >> 16) + ((xl*ah) >> 16);
   if (xl*al >= ~c + 1) c++;  /* thanks to Don Mitchell for this correction */
   return x;
   }

unsigned long Pref(unsigned long frekvence) {
return (unsigned long)((float)frekvence*74.09886448979591836734693877551*1.088*0.952/*/(1040.97)*65536.0*1.177*/);
}

//generator obecnych prubehu - sin, triangle, sawtooth, obdelnik, obec. prubeh nahrany z PC
void DDSGen(unsigned char mempos);
void DDSGen(unsigned char mempos) {
 //moznosti generatoru: 
 //prednastaveni urovni: 3V, 5V, 1V AC, 0,775V AC
 //metoda nastaveni: frekvence 
 //moznosti rozmitani: max. napeti na vystupu pro osu X, fmin, rozsah
 //meric charakteristit: vystup: pila, vystup 2: urceny pocet urovni
 //znacka pri rozmitani: na dane frekvenci setrva 10x dele (vyssi jas)
 
#define SPD_SIN 	0
#define SPD_TRI 	1
#define SPD_SAW 	2
#define SPD_SAWR	3
#define SPD_RECT	4
#define SPD_WHITE	5
#define SPD_PINK	6
#define SPD_232 	7

#define URD_3V		0
#define URD_5V		1
#define URD_1V		2
#define URD_0V7 	3
#define URD_SET 	4

#define DD_EEPROM_POS	(200+mempos)

 unsigned char prubeh=SPD_SIN,pom=0;
 unsigned int	voltage_low=0, voltage_high=4000;
 //unsigned long ;
 unsigned long register i, incr;
 unsigned long frekvence=1000,frekvenceroz=1000,incrr,frekvencezn;
 unsigned int vol_low,vol_high;
 unsigned int sinar[256];
 filter_on();
 do { 
 //nastaveni
 // 1. krok: nastaveni typu generatoru 
 prubeh=eeprom_read_byte(DD_EEPROM_POS);
  do {
  
   lcd_clear();
   printf_P(PSTR("Typ:"));
   lcd_putcommand(0xC0);
   if (prubeh==SPD_SIN) {printf_P(PSTR("SIN"));};
   if (prubeh==SPD_TRI) {printf_P(PSTR("TRI"));};
   if (prubeh==SPD_SAW) {printf_P(PSTR("SAW /|/|"));};
   if (prubeh==SPD_SAWR) {printf_P(PSTR("SAW rev"));};
   if (prubeh==SPD_RECT) {printf_P(PSTR("RECT"));};
   if (prubeh==SPD_WHITE) {printf_P(PSTR("White noise"));};
   if (prubeh==SPD_PINK) {printf_P(PSTR("Pink noise"));};  
   if (prubeh==SPD_232) {printf_P(PSTR("Zadat po RS232"));};
   if (TL_PLUS&&prubeh<7) {prubeh++;};
   if (TL_MINUS&&prubeh) {prubeh--;};
   while (TL_PLUS||TL_MINUS) {};
   DelayMs(100);
  //}
  
 } while (!TL_OK&&!TL_ESC&&TL_DALSI); 
 while (TL_OK||TL_ESC) {};
 eeprom_write_byte(DD_EEPROM_POS,prubeh); 
 // 2. krok: nastaveni urovni 
 pom=eeprom_read_byte(DD_EEPROM_POS+1);
 do {
  lcd_clear();
  printf_P(PSTR("Urovn\x06:"));
  lcd_putcommand(0xC0);  
  if (pom==UR_3V) {printf_P(PSTR("CMOS 3V/TTL"));voltage_low=500;voltage_high=800;};
  if (pom==UR_5V) {printf_P(PSTR("CMOS 5V"));voltage_low=500;voltage_high=1000;};
  if (pom==UR_1V) {printf_P(PSTR("+/-1,0V"));voltage_low=400;voltage_high=600;};
  if (pom==UR_0V7) {printf_P(PSTR("+/-0,7V"));voltage_low=423;voltage_high=577;};
  if (pom==UR_SET) {printf_P(PSTR("nastavit"));if (voltage_low==423) {voltage_low=500;voltage_high=800;}};
  if (TL_PLUS&&pom<4) {pom++;};
  if (TL_MINUS&&pom) {pom--;};
  while (TL_PLUS||TL_MINUS) {};
  DelayMs(100);
 } while (!TL_OK&&!TL_ESC&&TL_DALSI); 
 eeprom_write_byte(DD_EEPROM_POS+1,pom); 
 while (TL_OK||TL_ESC) {};
 // 2b. krok - podrobne nastaveni urovni
 unsigned char darozsah;
 do {
  darozsah=DA_REZIM_05;
  if (pom==UR_SET) {
   lcd_clear();
   printf_P(PSTR("Spicka zaporna:"));
   voltage_low=GetInt(eeprom_read_word(DD_EEPROM_POS+2),GI_V,0);
   eeprom_write_word(DD_EEPROM_POS+2,voltage_low);
   printf_P(PSTR("Spicka kladna:"));
   voltage_high=GetInt(eeprom_read_word(DD_EEPROM_POS+4),GI_V,0);	 
   eeprom_write_word(DD_EEPROM_POS+4,voltage_high);
  }
  //prepocet urovni
  // 1. krok - urceni rozsahu vystupu (0-5/0-10V nebo +-5V)
 
  if (voltage_low<500) {darozsah|=DA_REZIM_55;};
  if (voltage_high<500) {darozsah|=DA_REZIM_55;};
 
  if (voltage_low>1000) {darozsah|=DA_REZIM_010;};
  if (voltage_high>1000) {darozsah|=DA_REZIM_010;};

 if (darozsah==DA_REZIM_010+DA_REZIM_55) {
   printf_P(PSTR("CHYBA max.v\x02stup"));
   lcd_putcommand(0xC0);
   printf_P(PSTR("-+5V NEBO 0..10V"));
   DelayMs(2000);
  }; 
  if (darozsah==0) {darozsah=DA_REZIM_010;};
 } while (darozsah==DA_REZIM_010+DA_REZIM_55&&TL_DALSI);
 // 2. krok - vypoctu hodnot pro DA
 SetDAR(darozsah); 
 
 vol_high=DAPrepocet(darozsah,voltage_high);
 vol_low =DAPrepocet(darozsah,voltage_low);
 
 
// vypocet prubehu

 
 
 
 
 /*
  unsigned int i=0,incr=0;
 
 lcd_putch('>');

 i=0;
 lcd_putch('>');
 incr=120;
 while (1) {
  DelayMs(1);
  i+=incr;  
  dasend(sinar[i>>8]);
 }; 
 */
 
#define R_NO	0
#define R_LIN	1
#define R_LOG	2
 unsigned char rtyp=R_NO;
 // 3. krok - nastaveni frekvence - z ni vypocitat krok
 if (prubeh!=SPD_WHITE&&prubeh!=SPD_PINK) {
  lcd_clear();
  printf_P(PSTR("Frekvence (min):"));
  //incr=65536;
  frekvence=GetInt(eeprom_read_word(DD_EEPROM_POS+6),GI_ING|GI_HZ,15000); 
  if (frekvence>10000) {presnost();};
  eeprom_write_word(DD_EEPROM_POS+6,frekvence);
  incr=(unsigned long)((float)frekvence*61.697532109474816757447380808285*1.088/*.98/(1040.97)*65536.0*/);
  lcd_clear();
  printf_P(PSTR("Rozmitani:"));
  rtyp=eeprom_read_byte(DD_EEPROM_POS+8);
  do {   
   lcd_putcommand(0xC0);
   if (rtyp==R_NO) {printf_P(PSTR("VYP"));};
   if (rtyp==R_LIN) {printf_P(PSTR("LIN"));};
   if (rtyp==R_LOG) {printf_P(PSTR("LOG"));};
   while (TL_PLUS||TL_MINUS) {};
   DelayMs(50);
   if (TL_PLUS&&rtyp<R_LOG) {
    rtyp++;
   };
   if (TL_MINUS&&rtyp) {
    rtyp--;
   };
   
  } while (!TL_OK&&!TL_ESC&&TL_DALSI); 
  eeprom_write_byte(DD_EEPROM_POS+8,rtyp); 
  while (TL_ESC||TL_OK) {DelayMs(50);};
  
  if (rtyp) {
   lcd_clear();
   printf_P(PSTR("fmax rozmitani:"));
   frekvenceroz=GetInt(eeprom_read_word(DD_EEPROM_POS+9),GI_ING|GI_HZ,30000); 
   eeprom_write_word(DD_EEPROM_POS+9,frekvenceroz);
   incr=Pref(frekvence);
   incrr=Pref(frekvenceroz) /*(unsigned long)((float)frekvenceroz*72.616887230179544079080088763365*/  /*/1040.97*65536.0*1.177*/ /*)*/-incr;
   
   printf_P(PSTR("f znacky:"));
   frekvencezn=GetInt(eeprom_read_word(DD_EEPROM_POS+11),GI_ING|GI_HZ,frekvenceroz);	 
   eeprom_write_word(DD_EEPROM_POS+11,frekvencezn);
  } else {frekvenceroz=frekvence;}; 
 }; 
 
 //unsigned int vol_high_c,vol_low_c;
 //vol_high_c=vol_high;
 //vol_low_c=vol_low;
 
filter_off();
if (prubeh==SPD_232) {
  lcd_clear();
  CommandDone();
  fprintf_P(seriak,PSTR("_SIGLOAD\n"));
  printf_P(PSTR("\x05""ek\x01m na PC"));
  lcd_putcommand(0xC0);
 } else {cekejte(); };
 voltage_low=vol_low;voltage_high=vol_high;
 do {
 
 if (prubeh==SPD_SIN&&frekvence>4000) {
  vol_low=voltage_low-(unsigned int)((voltage_high-voltage_low)*(float)(frekvence-4000)*(4.0/300000.0));
  vol_high=voltage_high+(unsigned int)((voltage_high-voltage_low)*(float)(frekvence-4000)*(4.0/300000.0));
  if (vol_high>4095) {vol_high=4095;}
  if (vol_low>vol_high) {vol_low=0;}
 } else {
  vol_low=voltage_low;vol_high=voltage_high;
 }
 
 for (i=0;i<256;i++) {
  //SINUS
  if (prubeh==SPD_SIN) {
   sinar[i]=(unsigned int)((vol_high+vol_low)*0.5+(vol_high-vol_low)*0.5*sin(i*0.5*0.0490625));   
   filter_on();
  }
  if (prubeh==SPD_TRI) {
   sinar[i]=(unsigned int)(vol_low+(vol_high-vol_low)*(i<128?i:256-i)/128.0);
   
  }
  if (prubeh==SPD_SAW) {
   sinar[i]=(unsigned int)(vol_low+(vol_high-vol_low)*i/256.0);
   
  }
  if (prubeh==SPD_SAWR) {
   sinar[i]=(unsigned int)(vol_low+(vol_high-vol_low)*(256-i)/256.0);
   
  }
  if (prubeh==SPD_RECT) {
   sinar[i]=(unsigned int)(i<128?vol_low:vol_high);
   
  }
  
  //232 prubeh
  if (prubeh==SPD_232) {
   while (!CmdFlag&&TL_DALSI&&!TL_ESC) {    
   };
   if (CmdFlag) {
    DecodeLF();
    sinar[i]=(unsigned int)from;
	SetDAR((unsigned char)length);
	fprintf_P(seriak,PSTR("OK\n"));
	CommandDone();
   };
  }
  if (!(i&15)) {lcd_putch('.');};
 }; 
 if (prubeh==SPD_232) {fprintf_P(seriak,PSTR("_SIGDONE\n"));} 

 i=0;


 //vypis na displej
 lcd_clear();
 printf_P(PSTR("== Gener\x01tor =="));
 lcd_putcommand(0xC0);
 if (frekvenceroz>frekvence) {
  filter_off();
  printf_P(PSTR("Rozm\x07t\x01n\x07 ZAP."));
  //rozmitani v 256 bodech, v kazdem setrvat 2 (?) ms
  unsigned char rdel=2,rpos,znpos;
  //vypocitej polohu znacky
  if (rtyp==R_LIN) {
   znpos=(unsigned char)((unsigned long)((256*(frekvencezn-frekvence))/(frekvenceroz-frekvence)));
  } else {
   unsigned long znin=(unsigned long)((float)frekvencezn/907.87/*1040.97*/*65536.0);
   znpos=(unsigned char)((float) 256.0*log10((float)znin/(float)incr)/log10((float)(incrr+incr)/(float)(incr)));
  }  
  TCCR0=0x6A;
  zkrac=0;  
  unsigned long aktin;
  unsigned long logar[64];
  for (aktin=0;aktin<64;aktin++) {   
   logar[aktin]=(unsigned long)((float)( pow(10,log10((float)incr)+(log10((float)(incrr+incr)/(float)(incr))*((float)aktin/63.0) ))));
  };
  
  aktin=incr;
  
  while (!TL_ESC) {
   //vypocet frekvence
   //linearni
   
   if (rtyp==R_LIN) {
    //aktin=incr+(unsigned long)((incrr*rpos)>>8);
	incr=aktin+(unsigned long)((incrr*rpos)>>8);
   } else {
    //aktin=logar[rpos>>2];
	incr=logar[rpos>>2];
   };	   
   rpos++;
   //nast. osy X
   OCR0=rpos;
   //nastav cekani
   if ((rpos&127)==127) {
    if (TL_PLUS&&rdel<0x0F) {rdel++;};
    if (TL_MINUS&&rdel>1) {rdel--;};
   };	   
   if (rpos==znpos) {
    seccnt=rdel<<4;
   } else {
    seccnt=rdel;
   };	
   register unsigned char ch;
   unsigned int value;
   //generujeme
     while (seccnt--) {	
	 
	i+=incr;/*     dasend(sinar[(unsigned char)((unsigned long)i>>16)&0xFF]);
	  */
	   /*asm volatile(
    "add %A0, %A1 \n\t" 
	"adc %B0, %B1 \n\t" 
	"adc %C0, %C1 \n\t" 
    :	"=r" (i):"r"(incr)
   );*/
//ch=i>>16;
   
   asm volatile("mov %0, %C1"
	     : "=r" (ch)
	     : "r" (i)
   );
   
   /* Start transmission */
   value = sinar[ch];  
   
   while(!(SPSR & (1<<SPIF)));
   PORTB|=16;
   PORTB&=(unsigned char)~16;
   SPDR = (DA_INP_BUF+DA_GAIN_2+DA_ON) | (value>>8);
   
   while(!(SPSR & (1<<SPIF)));
   SPDR = (unsigned char) value ;
     }	
  }
  zkrac=5;
  TCCR0=0x05;
 } else
 if (prubeh==SPD_WHITE||prubeh==SPD_PINK) {
  if (SPD_WHITE==prubeh) {filter_off();};
  cli();
  //generovani sumu
  unsigned int pomdelta;

  pomdelta=vol_high-vol_low;
  initrnd();
  if (prubeh==SPD_WHITE) {
   while (TL_DALSI&&!TL_ESC) {	 
    dasend((unsigned int)(vol_low+(((unsigned long)(pomdelta)*(rndget()&0xFFFF))>>16)));	
   };
  } else {
   signed long b0=0,b1=0,b2=0,white;
   unsigned short tmp;
   while (TL_DALSI&&!TL_ESC) {	 
    white=(signed long)(rndget()&0xFFFF)-32768;
    b0 = 32692 * b0 + white * 3246;
    b1 = 31556 * b1 + white * 9716;
    b2 = 18678 * b2 + white * 34495;
    tmp = (unsigned int)((signed int)32768+(signed int)((signed long)(b0 + b1 + b2 + white * 6056)));
    dasend((unsigned int)(vol_low+(((unsigned long)(pomdelta)*(tmp&0xFFFF))>>16)));	
   };
  };
  sei();
 } else {  
  
  unsigned int value;
  
  register unsigned char ch;  
	
  cli();
  
  //generovani
  printf_P(PSTR("%d Hz"),frekvence);
  
  
  while (1) {   
   i+=incr;
  /* asm volatile(
    "add %A0, %A1 \n\t" 
	"adc %B0, %B1 \n\t" 
	"adc %C0, %C1 \n\t" 
    :	"=r" (i):"r"(incr)
   );	*/
   //ch=i>>16;
   
   asm volatile("mov %0, %C1"
	     : "=r" (ch)
	     : "r" (i)
   );
   value=sinar[ch];
   /* Start transmission */
   while(!(SPSR & (1<<SPIF)));
   PORTB|=16;
   PORTB&=(unsigned char)~16;
   SPDR = (DA_INP_BUF+DA_GAIN_2+DA_ON) | (value>>8);   
   if (TL_ESC) {i=0;break;};
   if (TL_PLUS||TL_MINUS) {
     lcd_putcommand(0xC0); 	
	 if (TL_PLUS&&frekvence<16000) {
	  frekvence+=10;	  
	  if (frekvence>=1000) {frekvence+=90;frekvence=frekvence/100*100;};
	 }
     if (TL_MINUS&&frekvence>10) {
	  frekvence-=10;
	  if (frekvence>1000) {frekvence-=90;frekvence=frekvence/100*100;};
	 }
	 frekvenceroz=frekvence=frekvence/10*10;
	 printf_P(PSTR("%u Hz "),frekvence);
     eeprom_write_word(DD_EEPROM_POS+6,frekvence);
//     incr=(unsigned long)((float)frekvence/1040.97*65536.0);
       incr=(unsigned long)((float)frekvence*61.697532109474816757447380808285*1.088/*.98/(1040.97)*65536.0*/);
	 //zajistime si znovu nabeh sem
	 i=1;
	 break;

   }
   /* Wait for transmission complete */
   while(!(SPSR & (1<<SPIF)));
   SPDR = (unsigned char) value ;
  }
  sei();
  /*
  //smycka dava 925,65 Hz pri XTalu 16 MHz, optimalizaci 2 a INCR=65536
  while (TL_DALSI&&!TL_ESC) {	
   i+=incr;
   dasend(sinar[(unsigned char)((unsigned long)i>>16)&0xFF]);
  }
  */
 }
 } while (i);
 } while (Opakovat());
 sei();
}

void Voltmetr() {
 unsigned int napeti,sondal=800,sondah=2500;
 unsigned char zvuk=0,last1;
 rozsah0_polovina();
 rozsah1_polovina();
 while (TL_DALSI&&!TL_ESC) {
  if (TL_PLUS) {
   lcd_clear();
   printf_P(PSTR("Log.sonda CMOS"));
   sondal=1600;sondah=3300;
  }
  if (TL_MINUS) {
   lcd_clear();
   printf_P(PSTR("Log.sonda TTL"));
   sondal=800;sondah=2500;
  }
  if (TL_OK) {
   lcd_clear();
   printf_P(PSTR("Zvuk IN1: "));
   zvuk++;
   if (zvuk==6) {zvuk=0;};
   if (zvuk==0) {lcd_putch('-');};
   if (zvuk==1) {lcd_putch('L');};
   if (zvuk==2) {lcd_putch('H');};
   if (zvuk==3) {lcd_putch('L');lcd_putch('H');};
   if (zvuk==4) {lcd_putch('H');lcd_putch('L');};
   if (zvuk==5) {printf_P(PSTR("zmena"));};
  }
  while (TL_MINUS||TL_PLUS||TL_OK) {
   DelayMs(200);   
  }
  a2dSetChannel(2);
  DelayUs(5);
  a2dStartConvert();
  DelayUs(5);
  napeti=a2dConvert10bit();
  lcd_putcommand(0x80);
  napeti=(unsigned int)((unsigned long)napeti*5000/1023);
  printf_P(PSTR("Inp1: %.1d,%.3d V "),napeti/1000,napeti%1000);
  if (napeti>sondah) {
   
   lcd_putch('H'); 
   if (zvuk==2) {BeepHighMs(1);}
   if ((zvuk==3||zvuk==5)&&last1==0) {BeepHighMs(10);};
   last1=1;
  } else 
  if (napeti<sondal) {
   
   lcd_putch('L');
   if (zvuk==1) {BeepLowMs(1);}
   if ((zvuk==4||zvuk==5)&&last1==1) {BeepLowMs(10);};
   last1=0;
  } else {
   lcd_putch('?');
  };
  if (zvuk) {lcd_putch('*');};
  a2dSetChannel(3);
  DelayUs(5);
  a2dStartConvert();
  DelayUs(5);
  napeti=a2dConvert10bit();
  lcd_putcommand(0xC0);
  napeti=(unsigned int)((unsigned long)napeti*5000/1023);
  printf_P(PSTR("Inp2: %.1d,%.3d V "),napeti/1000,napeti%1000);
  if (napeti>sondah) {
   lcd_putch('H');   
  } else 
  if (napeti<sondal) {
   lcd_putch('L');
  } else {
   lcd_putch('?');
  };
  DelayMs(1);
  
 };
}

void Mer232() {
 unsigned char a2drep=0,nostop=0;
 lcd_clear();
 printf_P(PSTR("Obsluha po RS232\n"));
 fprintf_P(seriak,PSTR("_REMOTE READY\n"));
 CommandDone();
 TCCR0=0x6A;
 do {
  //cekani na prikaz
  if (CmdFlag) {
   //prikazy displeje a klavesnice + beeperu
   if (CmdBuf[0]=='L') {
    if (CmdBuf[1]=='C') {
	 lcd_clear();
	};
    if (CmdBuf[1]=='N') {
	 lcd_putcommand(0xC0);
	};
    if (CmdBuf[1]=='W') {
	 unsigned char i;
	 for (i=3;i<20;i++) {
	  lcd_putch(CmdBuf[i]);
	 };
	};
	if (CmdBuf[1]=='b') {
	 BeepLowMs(20);
	};
	if (CmdBuf[1]=='B') {
	 BeepHighMs(20);
	};
	if (CmdBuf[1]=='K') {
	 //cteni klavesnice
	 fprintf_P(seriak,PSTR("_KB %d\n"),PINC);
	};
    fprintf_P(seriak,PSTR("OK\n"));
   }
   //prikazy vstupu
   if (CmdBuf[0]=='I') {   
    if (CmdBuf[2]=='R') {
	 fprintf_P(seriak,PSTR("_I %d\n"),PINA);
	} else 
    if (CmdBuf[1]=='0') {
	 if (CmdBuf[2]=='P') {
	  rozsah0_primo();
	 };
	 if (CmdBuf[2]=='H') {
	  rozsah0_polovina();
	 };
	 if (CmdBuf[2]=='+') {
	  rozsah0_plus();
	 };
	} else {
	 if (CmdBuf[2]=='P') {
	  rozsah1_primo();
	 };
	 if (CmdBuf[2]=='H') {
	  rozsah1_polovina();
	 };
	 if (CmdBuf[2]=='+') {
	  rozsah1_plus();
	 };
	}
	fprintf_P(seriak,PSTR("OK\n"));
   }
   //prikazy AD
   if (CmdBuf[0]=='A') {
    if (CmdBuf[1]=='0') {
     a2dSetChannel(2);
	};
    if (CmdBuf[1]=='1') {
     a2dSetChannel(3);
	};
    if (CmdBuf[1]=='S') {
	 a2drep=0;
     a2dStartConvert();
     DelayMs(1);
     fprintf_P(seriak,PSTR("_%d\n"),a2dConvert10bit());
	}; 
	if (CmdBuf[1]=='R') {
	 a2drep=1;     
	 a2dStartConvert();
	}; 
	if (CmdBuf[1]=='r') {
	 a2drep=0;     
	};   
	fprintf_P(seriak,PSTR("OK\n"));
   }
   //prikazy DA
   if (CmdBuf[0]=='D') {
    if (CmdBuf[1]=='F') {
	 filter_on();
	}
    if (CmdBuf[1]=='f') {
	 filter_off();
	}
    //DA vystup
	if (CmdBuf[1]=='O') {
	 DecodeLF();
	 dasendAsync((unsigned short)from);
	 OCR0=(unsigned char)length;
	 fprintf_P(seriak,PSTR("OK\n"));
	};
	if (CmdBuf[1]=='R') {
	 DecodeLF();
	 SetDAR((unsigned char)from);
	 fprintf_P(seriak,PSTR("OK\n"));
	}
   }
   //prikazy DIO a dalsi funkce
   if (CmdBuf[0]=='O') {
    //DIO DDR, PORT
	if (CmdBuf[1]=='N') {
	 nostop=1;
	};
	if (CmdBuf[1]=='n') {
	 nostop=0;
	};
    
    //DIO DDR, PORT
	if (CmdBuf[1]=='O') {
	 DecodeLF();
	 DDRD=(unsigned char)length;
	 PORTD=(unsigned char)from;
	};
	if (CmdBuf[1]=='I') {
	 fprintf_P(seriak,PSTR("_IO %d\n"),PIND);
	};	
	//CITAC
	if (CmdBuf[1]=='C') {
	 TCCR0=0x05;
	 fprintf_P(seriak,PSTR("_CNT %d\n"),citac());
	 TCCR0=0x6A;
	};	
	fprintf_P(seriak,PSTR("OK\n"));
   }
   CommandDone();
  };
  //a2drep
  if (a2drep) {
   //priznak hotove konverze
   if (ADCSR&&(1<<ADSC)) {
    DelayUs(10);
    a2dStartConvert();
	fprintf_P(seriak,PSTR("_%d\n"),a2dConvert10bit());
   };
  };
 } while ((TL_DALSI&&(!TL_ESC))||nostop); 
 TCCR0=0x05;
 fprintf_P(seriak,PSTR("_REMOTE STOPPED\n"));
};

void Setup() {
 unsigned long cpf; 
 cpf=eeprom_read_word(1);
 if (cpf<1000||cpf>32000) {
  cpf=10000;
 };
 if (TL_OK) {
  lcd_clear();	
  BeepLowMs(200);
  printf_P(PSTR("Kalibrac. C [pF]"));
  cpf=(unsigned short)GetInt(cpf,GI_FULLPR,0);
  eeprom_write_word(1,cpf);
 }; 
 ckal=(float)1.0e-12*cpf;
};

void DCOut();
void DCOut() {
 unsigned char darozsah=0;
 unsigned int voltage_low=500;
 SetDAR(darozsah); 
 dasendAsync(0);
 filter_on();
 voltage_low=eeprom_read_word(500);
 do {
  //nacti napeti
  darozsah=DA_REZIM_05;  
  lcd_clear();
  printf_P(PSTR("Zadej napeti:"));
  voltage_low=GetInt(voltage_low,GI_V|GI_HZ,0);  
  //prepocet urovni  
  if (voltage_low<500) {darozsah|=DA_REZIM_55;} else
  {darozsah|=DA_REZIM_010;};
  
  SetDAR(darozsah); 
  dasendAsync(DAPrepocet(darozsah,voltage_low));
  lcd_clear();
  printf_P(PSTR("Nastaveno:"));
  BeepHighMs(100);
  DelayMs(500);
 } while (!TL_ESC&&TL_DALSI); 
 eeprom_write_word(500,voltage_low);
};

int main(void) {
 //INICIALIZACE
 //porty
 DDRB=0xF9;
 DDRA=0xF3;
 PORTA=0x00;
 PORTB=0xF7;
 DDRC=0x03;
 PORTC=0xFC;
 DDRD=64+128+32+16;
 PORTD=0xFF;
 /* Enable SPI, Master, set clock rate fck */
 
 SPCR = (1<<SPE)|(1<<MSTR)/*|(1<<SPR0)*/;
 SPSR |= (1<<SPI2X);
 SPDR = 0;
 
 dasend(0);
 
 //nulu na vystup
 SetDAR(DA_REZIM_05);
 dasendAsync(0);
 
 //dame 100ms na ustaleni napeti...
 DelayMs(100); 
 //AD prevodnik
 a2dInit();
 //LCD displej
 lcd_init();
 lcd_setCZuserchars();
 lcd=fdevopen(lcd_putch, NULL);
 //seriovy kanal 
 uart_init(16); 
 seriak=fdevopen(uart_transmit, NULL);

 // Timer/Counter 0 initialization
 // Clock source: System Clock
 // Clock value: 62,500 kHz
 // Mode: Normal top=FFh
 // OC0 output: Disconnected
 TCCR0=0x04;
 TCNT0=0x05;
 OCR0=0x00;

 // Timer/Counter 1 initialization
 // Clock source: T1 pin Rising Edge
 // Mode: Normal top=FFFFh
 // OC1A output: Discon.
 // OC1B output: Discon.
 // Noise Canceler: Off
 // Input Capture on Falling Edge
 TCCR1A=0x00;
 TCCR1B=0x07;
 TCNT1=0;

 //TCCR1B=0x02; pro vstup=2MHz
 //TCCR1B=0x01; pro vstup=16MHz

 // Timer(s)/Counter(s) Interrupt(s) initialization
 TIMSK=0x05; 
 sei();
 
 if (TL_PLUS) {Diag();};
 Setup();
 
 //informace na displej
 lcd_clear();
 kalibrace_off();
 lcd_putcommand(0x0C);
 printf_P(PSTR("LCm/Gener\x01tor1.4"));
 lcd_putcommand(0xC0);
 printf_P(PSTR("Strasil.net 2006")); 
 DelayMs(600);
 BeepLowMs(50);
 BeepHighMs(50);
 unsigned char mereni=MER_DDS;
 
 while (1) {
  //MENU  
  while (!TL_OK&&TL_DALSI) {
   lcd_clear();
   printf_P(PSTR("Funkce:"));
   lcd_putcommand(0xC0);
/*
#define MER_VOLT	0
#define MER_ELLYT	1
#define MER_232 	2
#define MER_VOUT	3
#define MER_TIMER	4
#define MER_OSC 	5
#define MER_CITAC	6
#define MER_PERIODA	7
#define MER_RPM 	8
#define MER_GEN 	9
#define MER_DDS 	10
#define MER_DDS_1 	11
#define MER_DDS_2 	12
#define MER_MAX 	12
*/
   if (mereni==MER_VOLT) {
    printf_P(PSTR("Voltmetr"));
   }

   if (mereni==MER_232) {
    printf_P(PSTR("Ovl\x01""d\x01""n\x07 z PC"));
   }
   if (mereni==MER_VOUT) {
    printf_P(PSTR("DC v\x02stup"));
   }
   if (mereni==MER_TIMER) {
    printf_P(PSTR("\x05""asova\x05"));
   }
 
   if (mereni==MER_OSC) {
    printf_P( PSTR("Rezon.fr."));
   };	
   if (mereni==MER_CITAC) {
    printf_P( PSTR("\x05\x04ta\x05"));
   };	
   if (mereni==MER_PERIODA) {
    printf_P( PSTR("Perioda"));
   };	
   if (mereni==MER_RPM) {
    printf_P( PSTR("Imp/min."));
   };	
   if (mereni==MER_GEN) {
    printf_P( PSTR("Impulsn\x04 gen."));
   };	
   if (mereni>=MER_DDS) {
    printf_P( PSTR("Gener\x01tor "));
	if (mereni>MER_DDS) {
	 lcd_putch('M');
	 lcd_putch('0'+mereni-MER_DDS);
	}
   };	
   if (TL_PLUS) {
    mereni++;
	BeepLowMs(50);
	if (mereni>MER_MAX) {mereni=0;}
	while (TL_PLUS) {};
   };
   if (TL_MINUS) {
    BeepLowMs(50);
    if (mereni) {
	 mereni--;	 
	} else {
	 mereni=MER_MAX;
	};	
	while (TL_MINUS) {};	
   };
   DelayMs(50);
  };
  while (TL_OK) {};
  lcd_clear();
  DelayMs(70);	
  BeepHighMs(50);
  
  if (!TL_DALSI) {
//MERENI INDUKCNOSTI A KONDENZATORU
    LCMetr();  
  } else {
//DALSI MERENI
  if (mereni==MER_GEN||mereni==MER_TIMER) {
   ObdGen(mereni==MER_TIMER);
  }
  if (mereni>=MER_DDS) {
   DDSGen((mereni-MER_DDS)<<4);
  }
  if (mereni==MER_VOLT) {
   Voltmetr();
  };
  if (mereni==MER_232) {
   Mer232();
  }
  if (mereni==MER_VOUT) {
   DCOut();
  }

  if ((mereni==MER_OSC)&&TL_DALSI) { 
//CITAC 10MHz (podprogram vyuzivany i merenim LC)
   while (!TL_ESC&&TL_DALSI) {
    lcd_putcommand(0x80);
    printf_P(PSTR("f="));
    unsigned long frekvence=citac();
    lcd_putcommand(0xC0);
    printf_P(PSTR("%luHz        "),frekvence);
   };
  };
    
  if ((mereni==MER_PERIODA||mereni==MER_RPM||mereni==MER_CITAC)&&TL_DALSI) {
   #define VV_KOMP	2
   #define VV_DIN0	0
   #define VV_DIN1	1
   unsigned char vvstup=VV_DIN0,vpom,beeper=0;
   TCCR1B=0x02;
   TCNT1=0;
   T1CNT_high=0;
   lcd_clear();
   printf_P(PSTR("Vstup:"));
   do {
    lcd_putcommand(0xC0);
	if (vvstup==VV_KOMP) {
	 printf_P(PSTR("LC  "));	 
	} else if (vvstup==VV_DIN1) {
	 printf_P(PSTR("IN 2"));
	} else {
	 printf_P(PSTR("IN 1"));
	};	
	if (vvstup&&TL_MINUS) {vvstup--;};
	if (vvstup<2&&TL_PLUS) {vvstup++;};
	while (TL_MINUS||TL_PLUS) { DelayMs(10); };
   } while (!TL_ESC&&!TL_OK&&TL_DALSI);
   BeepLowMs(50);
   rozsah0_primo();
   rozsah1_primo();
   if (vvstup==VV_DIN0) {
    vpom=8;
   } else {
    vpom=4;
   };
   //zablokovat oscilator
   DDRB|=2;
   PORTB&=~2;
   
   while (!TL_ESC&&TL_DALSI) {
    //cekame 1 cyklus
	lcd_putcommand(0x80);
	if (mereni!=MER_CITAC) {
     printf_P(PSTR("Perioda:"));
	} else	{
	 printf_P(PSTR("Frekvence:"));
	};
    if (vvstup==VV_KOMP) {
     while (!(PINB&4)&&!TL_ESC) {
	 };
     while (PINB&4&&!TL_ESC) {
	 };
	} else {
     while (!(PINA&vpom)&&!TL_ESC) {
	 };
     while (PINA&vpom&&!TL_ESC) {
	 };
	};	
	//merime	

	if (beeper&&lipom<4000) {/*
	 fp=0.5*(float)(TCNT1)+(0.5*256.0*256.0)*(float)T1CNT_high;	*/
	 	TCNT1=0;
	T1CNT_high=0;
/*	 	 lipom=(unsigned long)floor(600000000/fp);
     fprintf_P(seriak,PSTR("%d\n"),lipom);		
	 BeepHighMs(10);*/
	} else {
		TCNT1=0;
	T1CNT_high=0;
	};


    if (vvstup==VV_KOMP) {
     while (!(PINB&4)&&!TL_ESC) {
	 };
     while (PINB&4&&!TL_ESC) {
	 };
	} else {
     while (!(PINA&vpom)&&!TL_ESC) {
	 };
     while (PINA&vpom&&!TL_ESC) {
	 };
	};
	//cas v us
	fp=0.5*(float)(TCNT1)+(0.5*256.0*256.0)*(float)T1CNT_high;
	TCNT1=0;
	T1CNT_high=0;

	
	lcd_putcommand(0xC0);
	if (mereni==MER_PERIODA) {
	 lipom=(unsigned long)floor(fp);	
	 if (lipom>1000000) {
	  printf_P(PSTR("%ld,%l.3d s"),(signed long)lipom/1000000,((signed long)lipom%1000000)/1000);
	 } else {
	  if (lipom>1000) {
	   printf_P(PSTR("%ld,%l.3d ms"),(signed long)lipom/1000,(signed long)lipom%1000);
	  } else {
	   printf_P(PSTR("%ld us"),(signed long)lipom);
	  };	  
     };
	 DelayMs(250);
	} else if (mereni==MER_CITAC) {
	 lipom=(unsigned long)floor(100000000/fp);	 
	 if (fp>1000) {
	  printf_P(PSTR("%l4d,%l.2d Hz "),(signed long)lipom/100,(signed long)lipom%100);	  	  
	  DelayMs(250);
	 } else {
	  
	  unsigned long fr=0;
      TCNT0=0;
      seccnt=249;
	  if (vvstup==VV_KOMP) {
       while (seccnt) {
	 while (!(PINB&4)&&seccnt) {
	     };
	 while (PINB&4&&seccnt) {
	     };
		 fr++;
	   };
	  } else {
	   while (seccnt) {
	while (!(PINA&vpom)&&seccnt) {
	};
	while (PINA&vpom&&seccnt) {
	    };
		fr++;
	   };	
      }   
	  fr=(unsigned long)(fr);
	  printf_P(PSTR("%l4d,%l.3d kHz"),(signed long)fr/1000,(signed long)fr%1000);	  	  
     }	 
	 
	} else {
	 lipom=(unsigned long)floor(600000000/fp);
	 if (lipom<500000) {
	  printf_P(PSTR("%ld,%l.1d/min"),(signed long)lipom/10,(signed long)lipom%10);	  	  
	 } else {
	  printf_P(PSTR("mimo rozsah"));
	 };	 
	}; 
	if (TL_PLUS) {beeper=1;};
	if (TL_MINUS) {beeper=0;};
	if (beeper&&lipom<4000) {BeepHighMs(40);fprintf_P(seriak,PSTR("%d\n"),lipom);} else {DelayMs(250);};
	
	lcd_putch(' ');lcd_putch(' ');lcd_putch(' ');lcd_putch(' ');lcd_putch(' ');lcd_putch(' ');lcd_putch(' ');lcd_putch(' ');
   };
   DDRB&=~2;
   TCCR1B=0x07;
  };
 };
 }; 
}
